""" Subclass for string.Template: adds placeholders() method. """
import string


__version__ = '2017.10.17'


class Template(string.Template):
    """
    Very thin subclass of ``string.Template``.
    """
    def placeholders(self, allow_repeats=False):
        """
        Return the names of the template variables in the template. Note that
        this returns a generator.

        The ``allow_repeats`` argument will cause the iteration to skip any
        repeated variable names.

        :type allow_repeats: bool
        :return: A generator of all the named vars
        :rtype: generator
        """
        seen = set()
        for match in self.pattern.finditer(self.template):
            named = match.group('named')
            if not allow_repeats:
                if named in seen:
                    continue
                else:
                    # Structured like this so that we don't incur the memory
                    # cost of storing previously-seen items when repeats are
                    # allowed.
                    seen.add(named)
            if named is None:
                # Seems to be necessary for things like "$(blah)".
                continue
            yield named



from cjrh_template import Template


def test_basic():
    s = '$person1 gave $thing to $person2'
    tmpl = Template(s)
    assert set(tmpl.placeholders()) == {'person1', 'thing', 'person2'}


def test_repeat_set():
    s = '$person1 gave $thing to $person2. $person1 was happy'
    tmpl = Template(s)
    assert set(tmpl.placeholders()) == {'person1', 'thing', 'person2'}


def test_repeat_list():
    s = '$person1 gave $thing to $person2. $person1 was happy'
    tmpl = Template(s)
    assert list(tmpl.placeholders()) == ['person1', 'thing', 'person2']


def test_repeat_list_repeat():
    s = '$person1 gave $thing to $person2. $person1 was happy'
    names = ['person1', 'thing', 'person2', 'person1']
    tmpl = Template(s)
    assert list(tmpl.placeholders(allow_repeats=True)) == names


def test_empty():
    s = 'This has\nno\nplaceholders\nwhatsoever'
    tmpl = Template(s)
    assert list(tmpl.placeholders()) == []


def test_makefile():
    s = '$(shell find $(SRC_DIRS))'
    tmpl = Template(s)
    assert list(tmpl.placeholders()) == []



#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup


setup(name='cjrh_template',
      version='2017.10.17',
      description='Subclass for string.Template: adds placeholders() method. ',
      author='Caleb Hattingh',
      author_email='caleb.hattingh@gmail.com',
      url='https://github.com/cjrh/cjrh_template',
      py_modules=['cjrh_template'],
     )
